/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/

package net.sf.jguard.ext.authentication.loginmodules;

import java.io.IOException;
import java.util.Locale;
import java.util.Map;
import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.LanguageCallback;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;
import net.sf.jguard.core.authentication.AuthenticationException;
import net.sf.jguard.core.authentication.callbacks.InetAddressCallback;
import net.sf.jguard.core.authentication.manager.AuthenticationManager;
import net.sf.jguard.core.authentication.manager.AuthenticationManagerFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Audit authentication success, failures, and logout events.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 */
public class AuditLoginModule implements LoginModule{

    private CallbackHandler cbh = null;
    private static final Logger logger = LoggerFactory.getLogger(AuditLoginModule.class.getName());
    private InetAddressCallback inetCbk;
    private NameCallback nameCallback;
    private Locale locale;
    private LanguageCallback languageCallback;
    public void initialize(Subject subject, CallbackHandler callbackHandler, Map<String, ?> sharedState, Map<String, ?> options) {
        cbh = callbackHandler;
    }

    public boolean login() throws LoginException {
        inetCbk = new InetAddressCallback();
        nameCallback = new NameCallback(" ");
        locale = Locale.getDefault();
        languageCallback = new LanguageCallback();

        //we define our needs by the callbacks array
        Callback[] callbacks = new Callback[3];
        callbacks[0]= inetCbk;
        callbacks[1]= nameCallback;
        callbacks[2] = languageCallback;
        try {
            cbh.handle(callbacks);
            inetCbk = (InetAddressCallback)callbacks[0];
            nameCallback = (NameCallback)callbacks[1];
            languageCallback = (LanguageCallback)callbacks[2];
            locale = languageCallback.getLocale();
        } catch (UnsupportedCallbackException ex) {
            logger.error(ex.getLocalizedMessage());
            return false;
        } catch (IOException ex) {
            logger.error(ex.getLocalizedMessage());
            return false;
        }
        return true;
    }

    public boolean commit() throws LoginException {
        return logAuthentication(true);
         
    }

    public boolean abort() throws LoginException {
        return logAuthentication(false);
    }

    public boolean logout() throws LoginException {
     AuthenticationManager authNManager = AuthenticationManagerFactory.getAuthenticationManager();
        try {
            long now = System.currentTimeMillis();
            authNManager.persistUserLogoutAttempt(nameCallback.getName(), inetCbk.getHostAdress(), inetCbk.getHostName(), now);
            logger.info("user from Host adress=" + inetCbk.getHostAdress() + " bound to host name=" + inetCbk.getHostName() + " has logoff" + " timeStamp=" + now + " locale=" + locale.getDisplayName());
        } catch (AuthenticationException ex) {
            logger.error( ex.getMessage(), ex);
            return false;
        }
        return true;
    }

    private boolean logAuthentication(boolean authenticationResult) throws LoginException {
        AuthenticationManager authNManager = AuthenticationManagerFactory.getAuthenticationManager();
        try {
            long now = System.currentTimeMillis();
            authNManager.persistUserLoginAttempt(nameCallback.getName(), inetCbk.getHostAdress(), inetCbk.getHostName(), authenticationResult, now);
            logger.info("user from Host adress=" + inetCbk.getHostAdress() + " bound to host name=" + inetCbk.getHostName() + " has tried to authenticate. boolean result =" + authenticationResult + " timeStamp=" + now + " locale=" + locale.getDisplayName());
            return true;
        } catch (AuthenticationException ex) {
            logger.error( ex.getMessage(), ex);
            return false;
        }
    }

}
