/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/

package net.sf.jguard.ext.authentication.loginmodules;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import javax.security.auth.Subject;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;

/**
 * provide a facade to aggregate use of some loginModules as only one.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 */
public class FacadeLoginModule implements LoginModule{
    private List loginModules = null;
    public final String LOGINMODULES = "loginmodules";
    /** Creates a new instance of NestedLoginModule */
    public FacadeLoginModule() {
        
    }

    public void initialize(Subject subject, CallbackHandler callbackHandler, Map sharedState, Map options) {
        
        loginModules = new ArrayList();
        String loginModulesClasses = (String)options.get(LOGINMODULES);
        List classNames = Arrays.asList(loginModulesClasses.split(","));

        Iterator itClasNames = classNames.iterator();
        while(itClasNames.hasNext()){
            String className = (String)itClasNames.next();
            Class clazz;
            try {
                clazz = Class.forName(className, true, Thread.currentThread().getContextClassLoader());
                LoginModule lm = (LoginModule)clazz.newInstance();
                loginModules.add(lm);
            } catch (ClassNotFoundException ex) {
                ex.printStackTrace();
            } catch (InstantiationException ex) {
                ex.printStackTrace();
            } catch (IllegalAccessException ex) {
                ex.printStackTrace();
            }
        }
        
        //build the options map dedicated to each nested loginmodule
        Iterator itLoginModules = loginModules.iterator();
        while(itLoginModules.hasNext()){
            LoginModule lm = (LoginModule)itLoginModules.next();
            Set entries = options.entrySet();
            Iterator itEntries = entries.iterator();
            Map lmOptions = new HashMap();
            String className = lm.getClass().getName();
            //from the global Map,we create a specific map for each nested loginmodule
            while(itEntries.hasNext()){
                Map.Entry entry = (Entry) itEntries.next();
                if(entry.getKey().equals(className)){
                    lmOptions.put(entry.getKey(),entry.getValue());
                }
            }
            lm.initialize(subject,callbackHandler,sharedState,lmOptions);
        }
    }

    public boolean login() throws LoginException {
        Iterator itLoginModules = loginModules.iterator();
        while(itLoginModules.hasNext()){
            LoginModule lm = (LoginModule)itLoginModules.next();
            lm.login();
        }
        return true;
    }

    public boolean commit() throws LoginException {
        Iterator itLoginModules = loginModules.iterator();
        while(itLoginModules.hasNext()){
            LoginModule lm = (LoginModule)itLoginModules.next();
            lm.commit();
        }
        return true;
    }

    public boolean abort() throws LoginException {
             Iterator itLoginModules = loginModules.iterator();
        while(itLoginModules.hasNext()){
            LoginModule lm = (LoginModule)itLoginModules.next();
            lm.abort();
        }
        return true;
    }

    public boolean logout() throws LoginException {
        Iterator itLoginModules = loginModules.iterator();
        while(itLoginModules.hasNext()){
            LoginModule lm = (LoginModule)itLoginModules.next();
            lm.logout();
        }
        return true;
    }
    
}
 